class DummyLikelihoodModule(object):
    """
    Dummy object for calculating a likelihood
    """

    def __init__(self):
        """
        Constructor of the DummyLikelihoodModule
        """
        pass
    
    def computeLikelihood(self, ctx):
        """
        Computes the likelihood using information from the context
        """
        # Get information from the context. This can be results from a core
        # module or the parameters coming from the sampler
        squares = ctx.get('squares_key')
        
        # Calculate a likelihood up to normalization
        lnprob = -sum(squares)/2.0
        
        # Return the likelihood
        return lnprob
    
    def setup(self):
        """
        Sets up the likelihood module.
        Tasks that need to be executed once per run
        """
        #e.g. load data from files
        
        print("DummyLikelihoodModule setup done")
            
        
    



import numpy as np


class FlatPositionGenerator(object):
    """
        Generates samples in a flat random space using
        center + (random values * 2 -1) * width
    """
    
    def __init__(self):
        """
            default constructor
        """
        pass

    def setup(self, sampler):
        """
            setup the generator
        """
        self.sampler = sampler
    
    def generate(self):
        """
            generates the positions
        """
        
        return [self.sampler.paramValues+(np.random.rand(self.sampler.paramCount)*2-1)*self.sampler.paramWidths for i in range(self.sampler.nwalkers)]
    
    def __str__(self, *args, **kwargs):
        return "FlatPositionGenerator"


##!/usr/bin/env python
"""
Test the CosmoHammerSampler module.

Execute with py.test -v

"""
from cosmoHammer.util.InMemoryStorageUtil import InMemoryStorageUtil

import numpy as np

class TestCosmoHammerSampler(object):
    storageUtil = None
    samples = np.array([[1,2,3],[4,5,6]])
    prob = np.array([1,1])
    
    def setup(self):
        self.storageUtil=InMemoryStorageUtil()
        
    def test_no_data(self):
        
        assert self.storageUtil.samplesBurnin == None
        assert self.storageUtil.probBurnin == None
    
    def test_persistBurninValues(self):
        self.storageUtil.persistBurninValues(self.samples, self.prob, [])
        
        assert self.storageUtil.samplesBurnin is not None
        assert self.storageUtil.probBurnin is not None
        assert self.storageUtil.samplesBurnin.shape == (2, 3)
        assert self.storageUtil.probBurnin.shape == (2, )
    
        self.storageUtil.persistBurninValues(self.samples, self.prob, [])
        
        assert self.storageUtil.samplesBurnin is not None
        assert self.storageUtil.probBurnin is not None
        assert self.storageUtil.samplesBurnin.shape == (4, 3)
        assert self.storageUtil.probBurnin.shape == (4, )
    
    def test_persistSamplingValues(self):
        self.storageUtil.persistSamplingValues(self.samples, self.prob, [])
        
        assert self.storageUtil.samples is not None
        assert self.storageUtil.prob is not None
        assert self.storageUtil.samples.shape == (2, 3)
        assert self.storageUtil.prob.shape == (2, )
    
        self.storageUtil.persistSamplingValues(self.samples, self.prob, [])
        
        assert self.storageUtil.samples is not None
        assert self.storageUtil.prob is not None
        assert self.storageUtil.samples.shape == (4, 3)
        assert self.storageUtil.prob.shape == (4, )
    



from __future__ import print_function, division, absolute_import, unicode_literals
import numpy as np
from collections import deque
import os

from cosmoHammer.ChainContext import ChainContext
from cosmoHammer.exceptions import LikelihoodComputationException
from cosmoHammer import getLogger
from cosmoHammer.util import Params

class LikelihoodComputationChain(object):
    """
    Implementation of a likelihood computation chain.
    """

    def __init__(self, min=None, max=None):
        """
        Constructor for the likelihood chain

        :param min: array 
            lower bound for the parameters
        :param max: array
            upper bound for the parameters
        """
        self.min = min
        self.max = max
        self._likelihoodModules = deque();
        self._coreModules = deque();
            

    def getCoreModules(self):
        """pointer to the likelihood module list """
        return self._coreModules

    def getLikelihoodModules(self):
        """pointer to the core module list """
        return self._likelihoodModules

    def addLikelihoodModule(self, module):
        """
        adds a module to the likelihood module list
        
        :param module: callable
            the callable module to add for the likelihood computation
        """
        self.getLikelihoodModules().append(module)
        
    def addCoreModule(self, module):
        """
        adds a module to the likelihood module list
        
        :param module: callable
            the callable module to add for the computation of the data
        """
        self.getCoreModules().append(module)
        
        
    def isValid(self, p):
        """
        checks if the given parameters are valid 
        """
        if(self.min is not None):
            for i in range(len(p)):
                if (p[i]<self.min[i]):
                    getLogger().debug("Params out of bounds i="+str(i)+" params "+str(p))
                    return False
        
        if(self.max is not None):
            for i in range(len(p)):
                if (p[i]>self.max[i]):
                    getLogger().debug("Params out of bounds i="+str(i)+" params "+str(p))
                    return False
        
        return True

    
    def setup(self):
        """sets up the chain and its modules """
        for cModule in self.getCoreModules():
            cModule.setup()
            
        for cModule in self.getLikelihoodModules():
            cModule.setup()
            
    
    def __call__(self, p):
        """
        Computes the log likelihood by calling all the core and likelihood modules.
        
        :param p: the parameter array for which the likelihood should be evaluated
        
        :return: the current likelihood and a dict with additional data
        """
        try:
            getLogger().debug("pid: %s, processing: %s"%(os.getpid(), p))
            if not self.isValid(p):
                raise LikelihoodComputationException()
            
            ctx = self.createChainContext(p)
    
            self.invokeCoreModules(ctx)
    
            likelihood = self.computeLikelihoods(ctx)
            getLogger().debug("pid: %s, processed. Returning: %s"%(os.getpid(), likelihood))
            return likelihood, ctx.getData()
        except LikelihoodComputationException:
            getLogger().debug("pid: %s, processed. Returning: %s"%(os.getpid(), -np.inf))
            return -np.inf, []
    
    def createChainContext(self, p):
        """
        Returns a new instance of a chain context 
        """
        try:
            p = Params(*zip(self.params.keys, p))
        except Exception:
            # no params or params has no keys
            pass
        return ChainContext(self, p)
    
    def invokeCoreModules(self, ctx):
        """
        Iterates thru the core modules and invokes them
        """
        for cModule in self.getCoreModules():
            self.invokeCoreModule(cModule, ctx)
            
    
    def invokeCoreModule(self, coreModule, ctx):
        """
        Invokes the given module with the given ChainContext
        """
        coreModule(ctx)
        
    
    def computeLikelihoods(self, ctx):
        """
        Computes the likelihoods by iterating thru all the modules.
        Sums up the log likelihoods.
        """
        likelihood = 0
        
        for lModule in self.getLikelihoodModules():
            likelihood += self.invokeLikelihoodModule(lModule, ctx)

        return likelihood
    
    def invokeLikelihoodModule(self, likelihoodModule, ctx):
        """
        Invokes the given module with the given ChainContext
        """
        return likelihoodModule.computeLikelihood(ctx)
    
    def __str__(self, *args, **kwargs):
        s = "Core Modules: \n  "
        s = s + "\n  ".join([type(o).__name__ for o in self.getCoreModules()])

        s = s + "\nLikelihood Modules: \n  "
        s = s + "\n  ".join([type(o).__name__ for o in self.getLikelihoodModules()])
        return s



##!/usr/bin/env python
"""
Test the TestSampleFileUtil module.

Execute with py.test -v

"""

from cosmoHammer.util import SampleBallPositionGenerator
from cosmoHammer.util import FlatPositionGenerator

class TestPositionGenerators(object):

    nwalkers = 10

    def setup(self):
        self.sampler = DummySampler([1,2], 2, [1,1], self.nwalkers)

    def test_SampleBallPositionGenerator(self):
        gen = SampleBallPositionGenerator()
        gen.setup(self.sampler)
        
        pos = gen.generate()
        assert pos is not None
        assert len(pos) == self.nwalkers
        
    def test_FlatPositionGenerator(self):
        gen = FlatPositionGenerator()
        gen.setup(self.sampler)
        
        pos = gen.generate()
        assert pos is not None
        assert len(pos) == self.nwalkers
        
class DummySampler(object):
    
    def __init__(self, paramValues, paramCount, paramWidths, nwalkers):
        self.paramValues = paramValues
        self.paramCount = paramCount
        self.paramWidths = paramWidths
        self.nwalkers = nwalkers


'''
Created on Oct 30, 2013

@author: J.Akeret
'''
from __future__ import print_function, division, absolute_import, \
    unicode_literals

from numpy.linalg.linalg import norm
from scipy.optimize.minpack import leastsq
from scipy.optimize import minimize
import numpy
import sys

def parabola(p, theta, thetabar=1):
    """
    Computation of the paraboloid for the given curvature matrix and samples.
    :param p: list of samples
    :param theta: vector containing the lower triangle of the matrix and the offset from the true mean
    
    :return: vector y from f(x,p)
    """
    
    leng, dim = theta.shape
    corrm, v, mu = transform(dim, p)
    
#     _cov = corr2cov(corrm, v)
#     R = numpy.linalg.inv(_cov)
    if(any(v==0)):
        vi = v
    else:
        vi = numpy.diag(1/v)
        
    R = numpy.dot(vi, numpy.dot(numpy.linalg.inv(corrm), vi))
    
    v = numpy.zeros(leng)
    for i,thetaj in enumerate(theta):
        thetaj = thetaj / thetabar
        v[i] = numpy.dot(thetaj.T,numpy.dot(R, thetaj)) #+ numpy.dot(thetaj,  mu)

    return numpy.array(v)


def errfunc(p,theta,delta, thetabar):
    """
    Error function defined by f(theta) - delta
    :param p: list of samples
    :param theta: the curvature matrix. see parabola def
    :param delta: the measured values
    """
    return parabola(p, theta, thetabar) - delta

def errfunc2(p,theta,delta, thetabar):
    """
    Error function defined by f(theta) - delta
    :param p: the curvature matrix. see parabola def
    :param theta: list of samples
    :param delta: the measured values
    """
    return sum((parabola(p, theta, thetabar) - delta)**2)

def transform(dim, p):
    """
    Transforms a vector containg the lower triangle of a matrix into a symmetric matrix
    
    :param p: the vector
    
    :return: the matrix and left over values
    """
    corrm = numpy.identity(dim)
    k=0
    for i in range(1,dim):
        for j in range(0,i):
            corrm[i,j]= p[k]
            k +=1
            
    corrm += corrm.T - numpy.diag(corrm.diagonal())
    
    vars = p[k:k+dim]
    mu = p[k+dim:]
    
    return corrm, vars, mu

def reverse(dim, R, vars):
    """
    Transforms a symmetric matrix into a vector containig the lower triangle
    
    :param R: the symmetric matrix
    
    :return: the vector
    """
    p = numpy.zeros(dim*(dim-1)/2)
    k=0
    for i in range(1,dim):
        for j in range(0,i):
            p[k] = R[i,j]
            k +=1
            
    p = numpy.append(p, vars)
    return numpy.append(p, numpy.zeros_like(vars))

def bound(x):
    dim = int(1./2 * (numpy.sqrt(8*len(x)+1)-1))
    _, stds, _ = transform(dim, x)
    return stds


class CurvatureFitter(object):
    '''
    Fits a paraboloid centered around the global best fit of the PSO by estimating a curvarture
    matrix with the particle given in the swarm
    
    :param swarm: list of particles
    :param gbest: the global best particle at the last iteration
    '''


    def __init__(self, swarm, gbest):
        '''
        Constructor
        '''
        self.swarm = swarm
        self.gbest = gbest
        
    def fit(self):
        """
        Fits the paraboloid to the swarm particles
        
        :return: the mean = global best position and the estimated covariance matrix
        """
        
        scale = 10**0
        dim = len(self.gbest.position) 
        
        x = numpy.array([particle.position * scale for particle in self.swarm])
        theta = (x - self.gbest.position * scale) #/ (self.gbest.position * scale)
        norms = numpy.array(list(map(norm, theta)))

        b = (norms < 0.1)
        theta = theta[b]
        fitness = numpy.array([particle.fitness * scale for particle in self.swarm])

        fitness =  fitness[b]
        delta = -2*(fitness - self.gbest.fitness * scale)
        
        _cov = self.minimize1(dim, theta, delta)
        _cov = self.minimize2(dim, theta, delta)

        return self.gbest.position, _cov
    
    def minimize1(self, dim, theta, delta):
        p0Cor = numpy.random.uniform(-1,1,dim**2).reshape(dim, dim)
        p0Cor = p0Cor - numpy.diag(p0Cor) + numpy.identity(dim)
        
        p0 = reverse(dim, numpy.identity(dim), numpy.ones(dim)/20)
        popt, _,infodict,mesg,_ = leastsq(errfunc, p0, args=(theta, delta, self.gbest.position),full_output=True)
        print(mesg)
         
         
        ss_err=(infodict['fvec']**2).sum()
        ss_tot=((delta-delta.mean())**2).sum()
        rsquared=1-(ss_err/ss_tot)
        print("rsquared", rsquared)
        
        corrm, var, mu = transform(dim, popt)
        var = var * self.gbest.position
        _cov = corr2cov(corrm, var)
        
        print("used mu:", mu)
        print("found _cov:\n", _cov)

        sigma = numpy.sqrt(numpy.diag(_cov))
        print( "=> found sigma:", sigma)
        
        return _cov
    
    def minimize2(self, dim, theta, delta):
        cons = (
         {'type': 'ineq',
          'fun' : lambda x: bound(x)})
        
        p0 = reverse(dim, numpy.identity(dim), numpy.ones(dim)*self.gbest.position/10)
        res = minimize(errfunc2, p0, args=(theta, delta, self.gbest.position), constraints=cons, method='SLSQP', options={'disp': True, "ftol":10**-17})
        popt=res.x

        corrm, var, mu = transform(dim, popt)
        var = var * self.gbest.position
        _cov = corr2cov(corrm, var)
        
        print("used mu:", mu)
        print("found _cov:\n", _cov)
        
        sigma = numpy.sqrt(numpy.diag(_cov))
        print( "=> found sigma:", sigma)

        return _cov

def corr2cov(corrm, var):
    dim = len(var)
    covm = numpy.empty((dim,dim))
    for i in range(len(corrm)):
        for j in range(len(corrm)):
            covm[i,j] = corrm[i,j]*var[i]*var[j]
            
    return covm

def rescale(_cov, v, dim):
        #rescaling
        cov2 = numpy.empty((dim, dim))
        for i in range(dim):
            for j in range(dim):
                cov2[i,j] = _cov[i,j] * v[i] * v[j]
        return cov2


'''
Created on Oct 28, 2013

@author: J.Akeret
'''
from __future__ import print_function, division, absolute_import, \
    unicode_literals
    
import multiprocessing
import numpy

from cosmoHammer.util.MpiUtil import MpiPool, mpiBCast
from cosmoHammer.pso.ParticleSwarmOptimizer import ParticleSwarmOptimizer



class MpiParticleSwarmOptimizer(ParticleSwarmOptimizer):
    """
    PSO with support for MPI to distribute the workload over multiple nodes
    """
    
    def __init__(self, func, low, high, particleCount=25, threads=1):
        self.threads = threads
        pool = MpiPool(self._getMapFunction())
        super(MpiParticleSwarmOptimizer, self).__init__(func, low, high, particleCount=particleCount, pool=pool)
        
        
    def _getMapFunction(self):
        if self.threads > 1:
            pool = multiprocessing.Pool(self.threads)
            return pool.map
        else:
            return map

    def _converged(self, it, p, m, n):
        
        if(self.isMaster()):
            converged =  super(MpiParticleSwarmOptimizer, self)._converged(it, p, m, n)
        else:
            converged = False
        
        converged = mpiBCast(converged)
        return converged
    
    def _get_fitness(self,swarm):
        mapFunction = self.pool.map
        
        mpiSwarm = mpiBCast(swarm)
        
        pos = numpy.array([part.position for part in mpiSwarm])
        results =  mapFunction(self.func, pos)
        lnprob = numpy.array([l[0] for l in results])
        for i, particle in enumerate(swarm):
            particle.fitness = lnprob[i]
            particle.position = pos[i]
    
    def isMaster(self):
        return self.pool.isMaster()



from cosmoHammer.util.SampleFileUtil import SampleFileUtil
from cosmoHammer.util.InMemoryStorageUtil import InMemoryStorageUtil
from cosmoHammer.util.SampleBallPositionGenerator import SampleBallPositionGenerator
from cosmoHammer.util.FlatPositionGenerator import FlatPositionGenerator
from cosmoHammer.util.Params import Params



from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np
from cosmoHammer import getLogger


#the real means..
WMAP7_MEANS = [70.704, 0.02256, 0.1115, 2.18474E-09, 0.9688, 0.08920]

# ...and non-trivial covariance matrix.
_cov  = np.array([[6.11E+00, 0, 0, 0, 0, 0],
                [7.19E-04, 3.26E-07, 0, 0, 0, 0],
                [-1.19E-02, -3.37E-07, 3.14E-05, 0, 0, 0],
                [-3.56E-11, 1.43E-14, 1.76E-13, 5.96E-21, 0, 0],
                [2.01E-02, 6.37E-06, -2.13E-05, 3.66E-13, 1.90E-04, 0],
                [1.10E-02, 2.36E-06, -1.92E-05, 8.70E-13, 7.32E-05, 2.23E-04]])
_cov += _cov.T - np.diag(_cov.diagonal())

# Invert the covariance matrix
WMAP7_ICOV = np.linalg.inv(_cov)


class PseudoCmbModule(object):
    """
    Chain for computing the likelihood of a multivariante gaussian distribution
    """
    def __init__(self, icov=WMAP7_ICOV, mu=WMAP7_MEANS, min_sz=0, max_sz=2):
        self.icov = icov
        self.mu = mu
        self.a = min_sz
        self.b = max_sz
    
    
    def computeLikelihood(self, ctx):
        x = ctx.getParams()

        diff = x[:6]-self.mu
        
        lnprob = -np.dot(diff,np.dot(self.icov,diff))/2.0

        lnprob -= np.log(self.b-self.a)
        return lnprob
    
    def setup(self):
        getLogger().info("Pseudo cmb setup")
            
        
    


#!/usr/bin/env python
"""
Runs CosmoHammer with the CambWrapper and WMAPWrapper.
The CosmoHammer will sample with 50*7 Walker for 2*250 iterations.

"""
from cosmoHammer import MpiCosmoHammerSampler
from cosmoHammer import LikelihoodComputationChain
from cosmoHammer.util import Params

from wmap9Wrapper import WmapExtLikelihoodModule as wmap9
from cambWrapper import CambCoreModule


#parameter start center, min, max, start width
params = Params(("hubble",                [70, 65, 80, 3]),
                ("ombh2",                 [0.0226, 0.01, 0.03, 0.001]),
                ("omch2",                 [0.122, 0.09, 0.2, 0.01]),
                ("scalar_amp",            [2.1e-9, 1.8e-9, 2.35e-9, 1e-10]),
                ("scalar_spectral_index", [0.96, 0.8, 1.2, 0.02]),
                ("re_optical_depth",      [0.09, 0.01, 0.1, 0.03]),
                ("sz_amp",                [1,0,2,0.4]))

chain = LikelihoodComputationChain(
                    min=params[:,1], 
                    max=params[:,2])

camb = CambCoreModule.CambCoreModule()

chain.addCoreModule(camb)

chain.addLikelihoodModule(wmap9.WmapExtLikelihoodModule())

chain.setup()


sampler = MpiCosmoHammerSampler(
                params= params, 
                likelihoodComputationChain=chain, 
                filePrefix="cosmoHammerWmap9_", 
                walkersRatio=20, 
                burninIterations=0, 
                sampleIterations=50)

print("start sampling")
sampler.startSampling()
print("done!")


#!/usr/bin/env python
"""
Runs CosmoHammer with a multivariant gaussian (50 dimensions).
"""

from cosmoHammer import CosmoHammerSampler
from cosmoHammer import LikelihoodComputationChain

from cosmoHammer.modules import MultivarianteGaussianModule

import numpy as np




# We'll sample a 4-dimensional Gaussian...
ndim = 4
# ...with randomly chosen mean position...
means = np.random.rand(ndim)

# ...and a positive definite, non-trivial covariance matrix.
cov  = 0.5-np.random.rand(ndim**2).reshape((ndim, ndim))
cov  = np.triu(cov)
cov += cov.T - np.diag(cov.diagonal())
cov  = np.dot(cov,cov)

# Invert the covariance matrix first.
icov = np.linalg.inv(cov)

params = [[0, -5, 5, 1] ]*ndim

params = np.array(params)

chain = LikelihoodComputationChain(params[:,1],params[:,2])

multivarianteGaussian = MultivarianteGaussianModule(icov, means)
chain.addLikelihoodModule(multivarianteGaussian)
chain.setup()

sampler = CosmoHammerSampler(
                params= params, 
                likelihoodComputationChain=chain, 
                filePrefix="multivariantGausian", 
                walkersRatio=50, 
                burninIterations=0, 
                sampleIterations=100,
                )

print("start sampling")
sampler.startSampling()
print("done!")


import logging

# Author:  Joel Akeret
# Contact: jakeret@phys.ethz.ch
"""
This is the CosmoHammer package.
"""

__version__   = '0.6.0'
__author__    = 'Joel Akeret'
__credits__   = 'Institute for Astronomy ETHZ, Institute of 4D Technologies FHNW'

def getLogger():
    return logging.getLogger(__name__)


from cosmoHammer.CosmoHammerSampler import CosmoHammerSampler
from cosmoHammer.MpiCosmoHammerSampler import MpiCosmoHammerSampler
from cosmoHammer.ConcurrentMpiCosmoHammerSampler import ConcurrentMpiCosmoHammerSampler

from cosmoHammer.LikelihoodComputationChain import LikelihoodComputationChain
from cosmoHammer.ChainContext import ChainContext

from cosmoHammer.pso.ParticleSwarmOptimizer import ParticleSwarmOptimizer
from cosmoHammer.pso.MpiParticleSwarmOptimizer import MpiParticleSwarmOptimizer



# Copyright (C) 2013 ETH Zurich, Institute for Astronomy

'''
Created on Mar 6, 2014

author: jakeret

Example of a file util that extends the functionallity of the 
standard ``SampleFileUtilSampleFileUtil`` in order to save additional parameters

'''
from cosmoHammer.util import SampleFileUtil

VALUE_1_KEY = "derived_params_key"
VALUE_2_KEY = "derived_params_key"

OUTPUT_ORDER = [VALUE_2_KEY, VALUE_1_KEY]

class DerivedParamterFileUtil(SampleFileUtil):
    '''
    Persists the derived parameters to a file
    '''

    def __init__(self, filePrefix, master=True, reuseBurnin=False):
        '''
        Constructor
        '''
        super(DerivedParamterFileUtil, self).__init__(filePrefix, master=master, reuseBurnin=reuseBurnin)
        
        self.paramsFile = open(self.filePrefix+"_params.dat", "w")
        
    def persistValues(self, posFile, probFile, pos, prob, data):
        #make sure the super class writes the position and likelihood to the disk
        super(DerivedParamterFileUtil, self).persistValues(posFile, probFile, pos, prob, data)
        
        #handle special parameters
        for derParams in data:
            if len(derParams) > 0:
                self.paramsFile.write("\t".join([str(derParams[key]) for key in OUTPUT_ORDER]))
            self.paramsFile.write("\n")
                                  
        self.paramsFile.flush();


##!/usr/bin/env python
"""
Test the CosmoHammerSampler module.

Execute with py.test -v

"""
import numpy as np
import os
import tempfile

from cosmoHammer import LikelihoodComputationChain
from cosmoHammer.modules.PseudoCmbModule import PseudoCmbModule

from cosmoHammer.util.IterationStopCriteriaStrategy import IterationStopCriteriaStrategy

from cosmoHammer.CosmoHammerSampler import CosmoHammerSampler

from cosmoHammer.util import SampleBallPositionGenerator
from cosmoHammer.util import FlatPositionGenerator
from cosmoHammer.util import InMemoryStorageUtil
from cosmoHammer.util import SampleFileUtil

class TestCosmoHammerSampler(object):
    params = None
    sampler = None
    
    def setup(self):
        """
        Initialise some data vectors used for the comparisons by the other functions.
        """
        self.params = np.array([[70, 40, 100, 3],
            [0.0226, 0.005, 0.1, 0.001],
            [0.122, 0.01, 0.99, 0.01],
            [2.1e-9, 1.48e-9, 5.45e-9, 1e-10],
            [0.96, 0.5, 1.5, 0.02],
            [0.09, 0.01, 0.8, 0.03],
            [1,0,2,0.4] ])


        #the real means..
        means = [70.704, 0.02256, 0.1115, 2.18474E-09, 0.9688, 0.08920]
        
        # ...and non-trivial covariance matrix.
        cov  = np.array([[6.11E+00, 0, 0, 0, 0, 0],
                        [7.19E-04, 3.26E-07, 0, 0, 0, 0],
                        [-1.19E-02, -3.37E-07, 3.14E-05, 0, 0, 0],
                        [-3.56E-11, 1.43E-14, 1.76E-13, 5.96E-21, 0, 0],
                        [2.01E-02, 6.37E-06, -2.13E-05, 3.66E-13, 1.90E-04, 0],
                        [1.10E-02, 2.36E-06, -1.92E-05, 8.70E-13, 7.32E-05, 2.23E-04]])
        cov += cov.T - np.diag(cov.diagonal())
        
        # Invert the covariance matrix
        icov = np.linalg.inv(cov)
        
        chain = LikelihoodComputationChain()
        pseudoLikelihood = PseudoCmbModule(icov, means)
        
        chain.addLikelihoodModule(pseudoLikelihood)
        chain.setup()
        
        posGen = FlatPositionGenerator()
        
        self.sampler = CosmoHammerSampler(
                        params= self.params, 
                        likelihoodComputationChain=chain, 
                        filePrefix=self._getTempFilePrefix(), 
                        walkersRatio=10, 
                        burninIterations=1, 
                        sampleIterations=11,
                        initPositionGenerator=posGen,
                        storageUtil=InMemoryStorageUtil())
        
    def test_init(self):
        self.sampler = CosmoHammerSampler(
                params= self.params, 
                likelihoodComputationChain=LikelihoodComputationChain(), 
                filePrefix=self._getTempFilePrefix(), 
                walkersRatio=10, 
                burninIterations=1, 
                sampleIterations=1)
        
        assert isinstance(self.sampler.storageUtil, SampleFileUtil) 
        assert isinstance(self.sampler.stopCriteriaStrategy, IterationStopCriteriaStrategy)
        assert isinstance(self.sampler.initPositionGenerator, SampleBallPositionGenerator)
        assert self.sampler.likelihoodComputationChain.params is not None

    
    def test_no_burn_in(self):
        self.sampler.resetSampler()
        self.sampler.burninIterations = 0;
        self.sampler.startSampling()
        
        assert self.sampler.storageUtil.samplesBurnin == None
        assert self.sampler.storageUtil.probBurnin == None
    
    def test_one_iter_burn_in(self):
        self.sampler.resetSampler()
        self.sampler.burninIterations = 1;
        self.sampler.walkersRatio = 10;
        self.sampler.startSampling()
        
        assert self.sampler.storageUtil.samplesBurnin is not None
        assert self.sampler.storageUtil.probBurnin is not None

        params = 7
        samples = params * self.sampler.burninIterations * self.sampler.walkersRatio
        assert self.sampler.storageUtil.samplesBurnin.shape == (samples, params)
        assert self.sampler.storageUtil.probBurnin.shape == (samples, )
    
    def test_one_iter_sampling(self):
        self.sampler.resetSampler()
        self.sampler.burninIterations = 0;
        self.sampler.sampleIterations = 1;
        self.sampler.walkersRatio = 10;
        self.sampler.startSampling()
        
        params = 7
        samples = params * self.sampler.sampleIterations * self.sampler.walkersRatio
        
        assert self.sampler.storageUtil.samplesBurnin is None
        assert self.sampler.storageUtil.probBurnin is None
        assert self.sampler.storageUtil.samples.shape == (samples, params)
        assert self.sampler.storageUtil.prob.shape == (samples, )
    
    def _getTempFilePrefix(self):
        return os.path.join(tempfile.mkdtemp(), "pseudoCmb")


from __future__ import print_function, division, absolute_import, unicode_literals

import emcee
import numpy as np
import logging
import time

import cosmoHammer
import cosmoHammer.Constants as c

from cosmoHammer import getLogger
from cosmoHammer.util import SampleFileUtil
from cosmoHammer.util import SampleBallPositionGenerator
from cosmoHammer.util.IterationStopCriteriaStrategy import IterationStopCriteriaStrategy



class CosmoHammerSampler(object):
    """
    A complete sampler implementation taking care of correct setup, chain burn in and sampling.

    :param params: the parameter of the priors
    :param likelihoodComputationChain: the callable computation chain
    :param filePrefix: the prefix for the log and output files
    :param walkersRatio: the ratio of walkers and the count of sampled parameters
    :param burninIterations: number of iteration for burn in
    :param sampleIterations: number of iteration to sample
    :param stopCriteriaStrategy: the strategy to stop the sampling. 
        Default is None an then IterationStopCriteriaStrategy is used
    :param initPositionGenerator: the generator for the init walker position. 
        Default is None an then SampleBallPositionGenerator is used
    :param storageUtil: util used to store the results
    :param threadCount: The count of threads to be used for the computation. Default is 1
    :param reuseBurnin: Flag if the burn in should be reused. 
        If true the values will be read from the file System. Default is False

    """

    def __init__(self, params, likelihoodComputationChain, filePrefix, walkersRatio, burninIterations, 
                 sampleIterations, stopCriteriaStrategy=None, initPositionGenerator=None, 
                 storageUtil=None, threadCount=1, reuseBurnin=False, logLevel=logging.INFO, pool=None):
        """
        CosmoHammer sampler implementation

        """
        self.params = params
        self.likelihoodComputationChain = likelihoodComputationChain
        self.walkersRatio = walkersRatio
        self.reuseBurnin = reuseBurnin
        self.filePrefix = filePrefix
        self.threadCount = threadCount
        self.paramCount = len(self.paramValues)
        self.nwalkers = self.paramCount*walkersRatio
        self.burninIterations = burninIterations
        self.sampleIterations = sampleIterations
        
        assert likelihoodComputationChain is not None, "The sampler needs a chain"
        assert sampleIterations > 0, "CosmoHammer needs to sample for at least one iterations"
        
        if not hasattr(self.likelihoodComputationChain, "params"):
            self.likelihoodComputationChain.params = params
        
        # setting up the logging
        self._configureLogging(filePrefix+c.LOG_FILE_SUFFIX, logLevel)
        
        if self.isMaster(): self.log("Using CosmoHammer "+str(cosmoHammer.__version__))
        
        # The sampler object
        self._sampler = self.createEmceeSampler(likelihoodComputationChain, pool=pool)
        
        if(storageUtil is None):
            storageUtil = self.createSampleFileUtil()
            
        self.storageUtil = storageUtil

        if(stopCriteriaStrategy is None):
            stopCriteriaStrategy = self.createStopCriteriaStrategy()
            
        stopCriteriaStrategy.setup(self)
        self.stopCriteriaStrategy = stopCriteriaStrategy
    
        if(initPositionGenerator is None):
            initPositionGenerator = self.createInitPositionGenerator()
            
        initPositionGenerator.setup(self)
        self.initPositionGenerator = initPositionGenerator
    
    
    def _configureLogging(self, filename, logLevel):
        logger = getLogger()
        logger.setLevel(logLevel)
        fh = logging.FileHandler(filename, "w")
        fh.setLevel(logLevel)
        # create console handler with a higher log level
        ch = logging.StreamHandler()
        ch.setLevel(logging.ERROR)
        # create formatter and add it to the handlers
        formatter = logging.Formatter('%(asctime)s %(levelname)s:%(message)s')
        fh.setFormatter(formatter)
        ch.setFormatter(formatter)
        # add the handlers to the logger
        for handler in logger.handlers[:]:
            logger.removeHandler(handler)
        logger.addHandler(fh)
        logger.addHandler(ch)
#         logging.basicConfig(format='%(asctime)s %(levelname)s:%(message)s', 
#                             filename=filename, filemode='w', level=logLevel)
        
        
    def createStopCriteriaStrategy(self):
        """
        Returns a new instance of a stop criteria stategy
        """
        return IterationStopCriteriaStrategy()
    
    def createSampleFileUtil(self):
        """
        Returns a new instance of a File Util
        """
        return SampleFileUtil(self.filePrefix, reuseBurnin=self.reuseBurnin)
    
    def createInitPositionGenerator(self):
        """
        Returns a new instance of a Init Position Generator
        """
        return SampleBallPositionGenerator()
    
    @property
    def paramValues(self):
        return self.params[:,0]
    
    @property
    def paramWidths(self):
        return self.params[:,3]
    
    def startSampling(self):
        """
        Launches the sampling
        """
        try:
            if self.isMaster(): self.log(self.__str__())
            if(self.burninIterations>0):
                
                if(self.reuseBurnin):
                    pos, prob, rstate = self.loadBurnin()
                    datas = [None]*len(pos)
                    
                else:
                    pos, prob, rstate, datas = self.startSampleBurnin()
            else:
                pos = self.createInitPos()
                prob = None
                rstate = None
                datas = None
            # Starting from the final position in the burn-in chain, sample for 1000
            # steps.
            self.log("start sampling after burn in")
            start = time.time()
            self.sample(pos, prob, rstate, datas)
            end = time.time()
            self.log("sampling done! Took: " + str(round(end-start,4))+"s")
    
            # Print out the mean acceptance fraction. In general, acceptance_fraction
            # has an entry for each walker
            self.log("Mean acceptance fraction:"+ str(round(np.mean(self._sampler.acceptance_fraction), 4)))
        finally:
            if self._sampler.pool is not None:
                try:
                    self._sampler.pool.close()
                except AttributeError:
                    pass
                try:
                    self.storageUtil.close()
                except AttributeError:
                    pass



    def loadBurnin(self):
        """
        loads the burn in form the file system
        """
        self.log("reusing previous burn in")

        pos = self.storageUtil.importFromFile(self.filePrefix+c.BURNIN_SUFFIX)[-self.nwalkers:]

        prob = self.storageUtil.importFromFile(self.filePrefix+c.BURNIN_PROB_SUFFIX)[-self.nwalkers:]

        rstate= self.storageUtil.importRandomState(self.filePrefix+c.BURNIN_STATE_SUFFIX)

        self.log("loading done")
        return pos, prob, rstate
    
    
    def startSampleBurnin(self):
        """
        Runs the sampler for the burn in
        """
        self.log("start burn in")
        start = time.time()
        p0 = self.createInitPos()
        pos, prob, rstate, data = self.sampleBurnin(p0)
        end = time.time()
        self.log("burn in sampling done! Took: " + str(round(end-start,4))+"s")
        self.log("Mean acceptance fraction for burn in:" + str(round(np.mean(self._sampler.acceptance_fraction), 4)))
        
        self.resetSampler()
        
        return pos, prob, rstate, data
    
    
    def resetSampler(self):
        """
        Resets the emcee sampler in the master node
        """
        if self.isMaster():
            self.log("Reseting emcee sampler")
            # Reset the chain to remove the burn-in samples.
            self._sampler.reset()        
    
    
    
    def sampleBurnin(self, p0):
        """
        Run the emcee sampler for the burnin to create walker which are independent form their starting position
        """
        
        counter = 1
        for pos, prob, rstate, datas in self._sampler.sample(p0, iterations=self.burninIterations):
            if self.isMaster():
                self.storageUtil.persistBurninValues(pos, prob, datas)
                if(counter%10==0):
                    self.log("Iteration finished:" + str(counter))
                
            counter = counter + 1

        if self.isMaster():
            self.log("storing random state")
            self.storageUtil.storeRandomState(self.filePrefix+c.BURNIN_STATE_SUFFIX, rstate)
            
        return pos, prob, rstate, datas


    def sample(self, burninPos, burninProb=None, burninRstate=None, datas=None):
        """
        Starts the sampling process
        """
        counter = 1
        for pos, prob, _, datas in self._sampler.sample(burninPos, lnprob0=burninProb, rstate0=burninRstate, 
                                                        blobs0=datas, iterations=self.sampleIterations):
            if self.isMaster():
                self.log("Iteration done. Persisting", logging.DEBUG)
                self.storageUtil.persistSamplingValues(pos, prob, datas)
                
                if(self.stopCriteriaStrategy.hasFinished()):
                    break
                
                if(counter%10==0):
                    self.log("Iteration finished:" + str(counter))
                
            counter = counter + 1


    def isMaster(self):
        """
        Returns True. Can be overridden for multitasking i.e. with MPI
        """
        return True

    def log(self, message, level=logging.INFO):
        """
        Logs a message to the logfile
        """
        getLogger().log(level, message)
    
    
    def createEmceeSampler(self, callable, **kwargs):
        """
        Factory method to create the emcee sampler
        """
        if self.isMaster(): self.log("Using emcee "+str(emcee.__version__))
        return emcee.EnsembleSampler(self.nwalkers, 
                                     self.paramCount, 
                                     callable, 
                                     threads=self.threadCount, 
                                     **kwargs)

    def createInitPos(self):
        """
        Factory method to create initial positions
        """
        return self.initPositionGenerator.generate()


    def getChain(self):
        """
            Returns the sample chain
        """
        return self._sampler.chain
    
    def __str__(self, *args, **kwargs):
        """
            Returns the string representation of the sampler config
        """
        desc = "Sampler: " + str(type(self))+"\n" \
                "configuration: \n" \
                "  Params: " +str(self.paramValues)+"\n" \
                "  Burnin iterations: " +str(self.burninIterations)+"\n" \
                "  Samples iterations: " +str(self.sampleIterations)+"\n" \
                "  Walkers ratio: " +str(self.walkersRatio)+"\n" \
                "  Reusing burn in: " +str(self.reuseBurnin)+"\n" \
                "  init pos generator: " +str(self.initPositionGenerator)+"\n" \
                "  stop criteria: " +str(self.stopCriteriaStrategy)+"\n" \
                "  storage util: " +str(self.storageUtil)+"\n" \
                "likelihoodComputationChain: \n" + str(self.likelihoodComputationChain) \
                +"\n"
        
        return desc



"""
Test the TestSampleFileUtil module.

Execute with py.test -v

"""
from __future__ import print_function, division, absolute_import, unicode_literals

import tempfile
import os
import numpy

import cosmoHammer.Constants as c
from cosmoHammer.util.SampleFileUtil import SampleFileUtil

class TestSampleFileUtil(object):

    prefix = "test"
    
    def createFileUtil(self):
        tempPath = tempfile.mkdtemp()
        tempPath = os.path.join(tempPath, self.prefix)
        fileUtil = SampleFileUtil(tempPath, True)
        return fileUtil, tempPath
    
    def test_not_master(self):
        tempPath = tempfile.mkdtemp()
        SampleFileUtil(tempPath, False)
        fileList = os.listdir(tempPath)
        assert len(fileList) == 0
        
        
    def test_persistBurninValues(self):
        fileUtil, tempPath = self.createFileUtil()
        
        pos = numpy.ones((10,5))
        prob = numpy.zeros(10)
        
        fileUtil.persistBurninValues(pos, prob, None)
        
        cPos = numpy.loadtxt(tempPath + c.BURNIN_SUFFIX)
        cProb = numpy.loadtxt(tempPath + c.BURNIN_PROB_SUFFIX)
        
        assert (pos == cPos).all()
        assert (prob == cProb).all()
        
        
    def test_persistSamplingValues(self):
        fileUtil, tempPath = self.createFileUtil()
        
        pos = numpy.ones((10,5))
        prob = numpy.zeros(10)
        
        fileUtil.persistSamplingValues(pos, prob, None)
        
        cPos = numpy.loadtxt(tempPath + c.FILE_SUFFIX)
        cProb = numpy.loadtxt(tempPath + c.PROB_SUFFIX)
        
        assert (pos == cPos).all()
        assert (prob == cProb).all()
        
    def test_importFromFile(self):
        fileUtil, tempPath = self.createFileUtil()
        
        pos = numpy.ones((10,5))
        prob = numpy.zeros(10)
        
        fileUtil.persistSamplingValues(pos, prob, None)
        
        cPos = fileUtil.importFromFile(tempPath + c.FILE_SUFFIX)
        cProb = fileUtil.importFromFile(tempPath + c.PROB_SUFFIX)
        
        assert (pos == cPos).all()
        assert (prob == cProb).all()
        
    def test_storeRandomState(self):
        fileUtil, tempPath = self.createFileUtil()
        
        rstate = numpy.random.mtrand.RandomState()
        fileUtil.storeRandomState(tempPath+c.BURNIN_STATE_SUFFIX, rstate)
        
        cRstate = fileUtil.importRandomState(tempPath+c.BURNIN_STATE_SUFFIX)
        
        print(rstate.get_state())
        oState = rstate.get_state()
        nState = cRstate.get_state()
        
        assert oState[0] == nState[0]
        assert all(oState[1] == nState[1])
        assert oState[2] == nState[2]
        assert oState[3] == nState[3]
        assert oState[4] == nState[4]
        


#!/usr/bin/env python
"""
Runs CosmoHammer with a rosenbrock density module
"""
from __future__ import print_function, division, absolute_import, unicode_literals

from cosmoHammer import CosmoHammerSampler
from cosmoHammer import LikelihoodComputationChain
from cosmoHammer.modules import RosenbrockModule

from cosmoHammer.util import Params

#parameter start center, min, max, start width
params = Params(("x", [1, -10, 10, 0.1]),
                ("y", [1, -10, 10, 0.1]))



chain = LikelihoodComputationChain()

rosenbrock = RosenbrockModule()

chain.addLikelihoodModule(rosenbrock)
chain.setup()


sampler = CosmoHammerSampler(
                params= params, 
                likelihoodComputationChain=chain, 
                filePrefix="rosenbrock", 
                walkersRatio=50, 
                burninIterations=100, 
                sampleIterations=100)
sampler.startSampling()

try:
    import numpy as np
    import matplotlib.pyplot as plt
    data = np.loadtxt("rosenbrock.out")
    plt.scatter(data[:,0], data[:, 1], 1)
    plt.xlabel(params.keys[0])
    plt.ylabel(params.keys[1])
    plt.show()
except Exception:
    print("Plotting failed. Missing libs?")


"""
Test the CosmoHammerSampler module.

Execute with py.test -v

"""
import numpy as np

from cosmoHammer.ChainContext import ChainContext
from cosmoHammer.pso.ParticleSwarmOptimizer import ParticleSwarmOptimizer
from cosmoHammer.pso.ParticleSwarmOptimizer import Particle

class TestCosmoHammerSampler(object):
    ctx = None
    params = np.array([[1,2,3],[4,5,6]])
    
    def setup(self):
        self.ctx=ChainContext(self, self.params)
        
    def test_Particle(self):
        particle = Particle.create(2)
        assert particle.fitness == -np.inf
        
        assert particle == particle.pbest
        
        particle2 = particle.copy()
        assert particle.fitness == particle2.fitness
        assert particle.paramCount == particle2.paramCount
        assert (particle.position == particle2.position).all()
        assert (particle.velocity == particle2.velocity).all()
        
        
        particle.fitness = 1
        particle.updatePBest()
        
        assert particle.pbest.fitness == 1        
    

    def test_setup(self):
        low = np.zeros(2)
        high = np.ones(2)
        pso = ParticleSwarmOptimizer(None, low, high, 10)
        
        assert pso.swarm is not None
        assert len(pso.swarm) == 10
        
        position = [part.position for part in pso.swarm]
        
        assert (position>=low).all()
        assert (position<=high).all()
        
        velocity = [part.velocity for part in pso.swarm]
        assert (velocity == np.zeros(2)).all()
        
        fitness = [part.fitness == 0 for part in pso.swarm]
        assert all(fitness)
        
        assert pso.gbest.fitness == -np.inf
        
        
    def test_optimize(self):
        low = np.zeros(2)
        high = np.ones(2)
        func = lambda p: (-np.random.rand(), None)
        pso = ParticleSwarmOptimizer(func, low, high, 10)
        
        maxIter=10
        swarms, gbests = pso.optimize(maxIter)
        assert swarms is not None
        assert gbests is not None
        assert len(swarms) == maxIter
        assert len(gbests) == maxIter
        
        fitness = [part.fitness != 0 for part in pso.swarm]
        assert all(fitness)
        
        assert pso.gbest.fitness != -np.inf 
        
        


#Created on Nov 11, 2013
#author: jakeret


class LikelihoodComputationException(Exception):
    '''
    Exception for likelihood computation
    '''
    def __init__(self):
        '''
        Constructor
        '''
        pass
    
class InvalidLikelihoodException(LikelihoodComputationException):
    """
    Exception for invalid likelihoods e.g. -loglike >= 0.0
    """
    
    def __init__(self, params=None):
        self.params = params
        



'''
Created on Oct 22, 2013

@author: J.Akeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals

import numpy
from cosmoHammer.pso.ParticleSwarmOptimizer import ParticleSwarmOptimizer
from cosmoHammer.pso.CurvatureFitter import CurvatureFitter

class BestFitPositionGenerator(object):
    '''
    A position generator which uses a particle swarm optimization algorithm 
    to find the best fit value and the collapsed swarm to estimate the curvature matrix
    at that point. The optimization process can be parallelized over
    MPI and python multiprocessing.
    
    :param mpi: True if a MPI implementation of the PSO should be used. Default is False 
    :param threads: Number of multiprocessing thread that should be started. Default is 1
    :param particleCount: Number of particle to use for the optimization. If none 
        the number is derrived according to the size of the parameter space. Default is None
    :param maxIter: the maximal number of iterations. Default will be set to MAX_PSO_ITER
    
    '''
    
    MAX_PSO_ITER = 1000
    
    MIN_PARTICLE_COUNT = 20
    
    BEST_FILE_NAME = "_best_fit_global.out"

    BEST_INFO_FILE_NAME = "_best_fit_info.out"

    def __init__(self, mpi=False, threads=1, particleCount=None, maxIter=None):
        """
            default constructor
        """
        self.mpi = mpi
        self.threads = threads
        self.particleCount = particleCount
        
        self.maxIter = maxIter
        if(self.maxIter is None):
            self.maxIter = self.MAX_PSO_ITER
        

    def setup(self, sampler):
        """
        setup the generator
        """
        self.sampler = sampler
        
    def generate(self):
        """
        generates the positions by running the PSO and using the chain's min and max and then calling 
        the paraboloid fitter in order to estimate the covariance matrix. The position will then
        be generated by drawing position from a multivariant gaussian distribution defined by
        the best fit and the estimated covariance matrix.
        The progress of the PSO is successively stored to a the disk.
        """
        
        chain = self.sampler.likelihoodComputationChain
        
        if(self.particleCount is None):
            self.particleCount = self.get_particle_count()

        if(self.mpi):
            #only import when needed in order to avoid an error in case mpi4py is not installed
            from cosmoHammer.sampler.util.pso.MpiParticleSwarmOptimizer import MpiParticleSwarmOptimizer
            
            pso = MpiParticleSwarmOptimizer(chain, chain.min, chain.max, self.particleCount, threads=self.threads)
        else:
            pso = ParticleSwarmOptimizer(chain, chain.min, chain.max, self.particleCount, threads=self.threads)
        
        swarm = []
        with open(self.sampler.filePrefix+self.BEST_FILE_NAME, "w") as f:
            for i, cswarm in enumerate(pso.sample(self.maxIter)):
                self._save(f, i, pso)
                if(i>=0):
                    swarm.append(cswarm)

            self._save(f, i+1, pso)
        self.sampler.log("Best fit found after %s iteration: %f %s"%(i+1, pso.gbest.fitness, pso.gbest.position))
        
        
        fswarm = []
        for i in range(1,5):
            fswarm += swarm[-i]

        self._storeSwarm(fswarm)
        
        fitter = CurvatureFitter(fswarm, pso.gbest)
        mean, _cov = fitter.fit()
        
        self._storeFit(pso.gbest, _cov)

#         dim = len(mean)-1
#         sigma = 0.4
#         factor = _cov[dim,dim] / numpy.sqrt(sigma)
#         _cov[:-1,dim] = _cov[:-1,dim]/factor
#         _cov[dim,:-1] = _cov[dim,:-1]/factor
#         _cov[dim,dim] = sigma
#         print ""
#         fitter = ParaboloidFitter(fswarm, pso.gbest, True)
#         mean, _cov = fitter.fit()
        sigma = numpy.sqrt(numpy.diag(_cov))
        print("=> found sigma:", sigma)
        
#        fitter = ParaboloidFitter(pso.swarm, pso.gbest)
#        mean, _cov = fitter.fit()
#        sigma = numpy.sqrt(numpy.diag(_cov))
#        print "=> found sigma:", sigma
        
        samples = numpy.random.multivariate_normal(mean, _cov, self.sampler.nwalkers)
#         print numpy.std(samples, axis=0)
        return samples
        
#         self.sampler.paramValues = pso.gbest.position
#         self.sampler.paramWidths = self.sampler.paramValues * self.SPREAD_FACTOR
#         generator = SampleBallPositionGenerator()
#         generator.setup(self.sampler)
#         return generator.generate()

        
        
    
    def get_particle_count(self):
        """
        Generates the number of particles to use by using a logarithmic function of the parameter count
        """
        return int(self.MIN_PARTICLE_COUNT + self.MIN_PARTICLE_COUNT*numpy.log(self.sampler.paramCount))
    
    def __str__(self, *args, **kwargs):
        return "BestFitPositionGenerator: particleCount=%s, mpi=%s, threads=%s"%(self.particleCount, self.mpi, self.threads)
    
    def _save(self, f, i, pso):
        if(pso.isMaster()):
            particle = pso.gbest
            f.write("%s\t%f\t"%(i, particle.fitness))
            f.write("\t".join([str(p) for p in particle.position]))
            f.write("\n")
            f.flush()
                        
    def _storeFit(self, gbest, _cov):
        with open(self.sampler.filePrefix+self.BEST_INFO_FILE_NAME, "w") as f:
            f.write("#Best fit: %s\n"%(gbest.fitness))
            f.write(", ".join([str(i) for i in gbest.position]))
            f.write("\n#Estimated covariance matrix:\n")
            for row in _cov:
                f.write ("[" + ",  ".join([str(i) for i in row]) + "]\n")

    def _storeSwarm(self, swarm):
        with open(self.sampler.filePrefix+"swarm", "w") as f:
            for particle in swarm:
                f.write(str(particle.fitness))
                f.write("\t")
                f.write("\t".join([str(p) for p in particle.position]))
                f.write("\n")


#!/usr/bin/env python
"""
Some constants used by the samplers

"""

FILE_SUFFIX = ".out"

LOG_FILE_SUFFIX = ".log"

PROB_PREFIX = "prob"

BURNIN_PREFIX = "burnin"

STATE_PREFIX = "state"

PROB_SUFFIX = PROB_PREFIX+FILE_SUFFIX

BURNIN_SUFFIX = BURNIN_PREFIX+FILE_SUFFIX

BURNIN_PROB_SUFFIX = BURNIN_PREFIX+PROB_PREFIX+FILE_SUFFIX

BURNIN_STATE_SUFFIX = BURNIN_PREFIX+STATE_PREFIX+FILE_SUFFIX



# Copyright (C) 2015 ETH Zurich, Institute for Astronomy

'''
Created on Aug 5, 2015

author: jakeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals

from copy import copy

import numpy as np


class Params(object):
    """
    A key-value parameter store preserving the order of the params passed to the intializer.

    Examples::
    
        $ params = Params(("key1", [1,2,3]),
                          ("key2", [1,2,3]))
    
        $ print(params.keys)
        > ['key1', 'key2']
        
        $ print(params.key1)
        > [1, 2, 3]
        
        $ params[:,0] = 0
        
        $ print(params.values)
        > [[0 2 3]
           [0 2 3]]
           
        $ print(params[:,1])
        > [2 2]
        
    """
    
    def __init__(self, *args):

        values = []
        self._keys = []
        for k,v in args:
            if k in self._keys:
                raise KeyError("Duplicated key '%s'"%k)
            
            self.__dict__[k] = v
            self._keys.append(k)
            values.append(v)
        self._values = np.array(values) 
        
    def __getitem__(self, slice):
        return self.values[slice]
    
    def __setitem__(self, slice, value):
        self.values[slice] = value
    
    def __str__(self):
        return ",".join(("%s=%s"%(k,v) for k,v in zip(self.keys, self.values)))
    
    @property
    def keys(self):
        return copy(self._keys)

    @property
    def values(self):
        return self._values
    
    def get(self, key):
        return self.__dict__[key]
    
    def copy(self):
        return Params(*zip(self.keys, self.values))




class IterationStopCriteriaStrategy(object):
    """
        A very simple implementation of a stop criteria strategy which does not stop the sampling until the max iteration is reached.
    """
    
    def __init__(self):
        """
            default constructor
        """
        pass
        
    def setup(self, sampler):
        """
            setup the strategy
        """
        self.sampler = sampler
        
    def hasFinished(self):
        """
            always returns false
        """
        return False

    def __str__(self, *args, **kwargs):
        return "IterationStopCriteriaStrategy"


##!/usr/bin/env python
"""
Test the LikelihoodComputationChain module.

Execute with py.test -v

"""

import numpy as np
from cosmoHammer import LikelihoodComputationChain
from cosmoHammer.util import Params

class TestLikelihoodComputationChain(object):

    def test_modules(self):
        chain = LikelihoodComputationChain()
        
        assert len(chain.getCoreModules())==0
        assert len(chain.getLikelihoodModules())==0
        
        coreModule = DummyModule()
        likeModule = DummyModule()
        chain.addCoreModule(coreModule)
        chain.addLikelihoodModule(likeModule)
        assert len(chain.getCoreModules())==1
        assert len(chain.getLikelihoodModules())==1
        
        chain.setup()
        assert coreModule.init
        assert likeModule.init
        
        like, data = chain([0])
        
        assert coreModule.called
        assert likeModule.compLike
        
        assert like == DummyModule.like
        assert len(data) == 1
        assert data["data"] == DummyModule.data
        
    def test_isValid(self):
        chain = LikelihoodComputationChain()
        assert chain.isValid([0])

        chain = LikelihoodComputationChain(min=[0])
        assert chain.isValid([1])
        assert chain.isValid([0])
        assert not chain.isValid([-1])

        chain = LikelihoodComputationChain(min=[0, 1])
        assert chain.isValid([1, 2])
        assert chain.isValid([0, 1])
        assert not chain.isValid([-1, 1])
        assert not chain.isValid([0, 0])
        assert not chain.isValid([-1, 0])

        chain = LikelihoodComputationChain(max=[1])
        assert chain.isValid([1])
        assert chain.isValid([0])
        assert not chain.isValid([2])

        chain = LikelihoodComputationChain(max=[1, 2])
        assert chain.isValid([0, 1])
        assert chain.isValid([1, 2])
        assert not chain.isValid([2, 2])
        assert not chain.isValid([1, 3])
        assert not chain.isValid([2, 3])

        chain = LikelihoodComputationChain(min=[0, 1], max=[1, 2])
        assert chain.isValid([1, 2])
        assert chain.isValid([0, 1])
        assert chain.isValid([0, 1])
        assert chain.isValid([1, 2])
        assert not chain.isValid([-1, 1])
        assert not chain.isValid([0, 0])
        assert not chain.isValid([-1, 0])
        assert not chain.isValid([2, 2])
        assert not chain.isValid([1, 3])
        assert not chain.isValid([2, 3])
        
        like, data = chain([-1, 0])
        assert like == -np.inf
        assert len(data) == 0
        
        like, data = chain([2, 3])
        assert like == -np.inf
        assert len(data) == 0
        
    def test_createChainContext(self):
        chain = LikelihoodComputationChain()
        
        p = np.array([1,2])
        ctx = chain.createChainContext(p)
        assert ctx is not None
        assert np.all(ctx.getParams() == p)

    def test_createChainContext_params_invalid(self):
        chain = LikelihoodComputationChain()
        chain.values = []
        
        p = np.array([1,2])
        ctx = chain.createChainContext(p)
        
        assert ctx is not None
        assert np.all(ctx.getParams() == p)
        
    def test_createChainContext_params(self):
        keys = ["a", "b"]
        params = Params((keys[0], 0),
                        (keys[1], 1))
        chain = LikelihoodComputationChain()
        chain.params = params
        
        p = np.array([1,2])
        ctx = chain.createChainContext(p)
        
        assert ctx is not None
        assert np.all(ctx.getParams().keys == keys) 
        assert np.all(ctx.getParams()[0] == p[0]) 
        assert np.all(ctx.getParams()[1] == p[1]) 
        

        
class DummyModule(object):
    
    like = 0
    data = 1
    
    def __init__(self):
        self.init = False
        self.called = False
        self.compLike = False
    
    def setup(self):
        self.init = True
        
    def __call__(self, ctx):
        self.called = True
        ctx.getData()["data"] = self.data
        
    def computeLikelihood(self, ctx):
        self.compLike = True
        return self.like


# Copyright (C) 2014 ETH Zurich, Institute for Astronomy

'''
Created on Jul 29, 2014

author: jakeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np
from mock import patch

from cosmoHammer.util import MpiUtil

class TestMpiUtil(object):

    @patch("cosmoHammer.util.MpiUtil.MPI")
    def test_splitlist_1(self, mpi_mock):
        sequence = self._get_sequence(7)
        n = 1
        sList = MpiUtil.splitList(sequence, n)
        assert len(sList) == n
        for i in range(len(sequence)):
            assert np.all(sList[0][i] == sequence[i])
            
    @patch("cosmoHammer.util.MpiUtil.MPI")
    def test_splitlist_2(self, mpi_mock):
        sequence = self._get_sequence(7)
        n = 2
        sList = MpiUtil.splitList(sequence, n)
        assert len(sList) == n

        assert len(sList[0]) == 4
        for i in range(4):
            assert np.all(sList[0][i] == sequence[0+i])
            
        assert len(sList[1]) == 3
        for i in range(3):
            assert np.all(sList[1][i] == sequence[4+i])
            
    @patch("cosmoHammer.util.MpiUtil.MPI")
    def test_splitlist_3(self, mpi_mock):
        sequence = self._get_sequence(7)
        n = 3
        sList = MpiUtil.splitList(sequence, n)
        assert len(sList) == n

        l0 = 2
        assert len(sList[0]) == l0
        for i in range(l0):
            assert np.all(sList[0][i] == sequence[0+i])
        
        l1 = 3    
        assert len(sList[1]) == l1
        for i in range(l1):
            assert np.all(sList[1][i] == sequence[2+i])
            
        l2 = 2
        assert len(sList[2]) == l2
        for i in range(l2):
            assert np.all(sList[2][i] == sequence[5+i])

    @patch("cosmoHammer.util.MpiUtil.MPI")
    def test_splitlist_equal(self, mpi_mock):
        sequence = self._get_sequence(10)
        
        n = 10
        sList = MpiUtil.splitList(sequence, n)
        assert len(sList) == n

        l0 = 1
        for k in range(n):
            assert len(sList[k]) == l0
            for i in range(l0):
                assert np.all(sList[k][i] == sequence[(k*l0)+i])

    @patch("cosmoHammer.util.MpiUtil.MPI")
    def test_splitlist_80(self, mpi_mock):
        sequence = self._get_sequence(160)
        
        n = 80
        sList = MpiUtil.splitList(sequence, n)
        assert len(sList) == n

        l0 = 2
        for k in range(n):
            assert len(sList[k]) == l0
            for i in range(l0):
                assert np.all(sList[k][i] == sequence[(k*l0)+i])
        

    def _get_sequence(self, lenght):
        sequence = (np.ones((lenght,4)).T * np.arange(lenght)).T
        sequence = [sequence[i] for i in range(len(sequence))]
        return sequence



PARENT_KEY = "key_parent"
PARAMS_KEY = "key_params"
DATA_KEY = "key_data"

class ChainContext(object):
    """
    Context holding a dict to store data and information durring the computation of the likelihood
    """
    
    def __init__(self, parent, params):
        """
        Constructor of the context
        """
        
        self._data = dict()
        self.add(PARENT_KEY, parent)
        self.add(PARAMS_KEY, params)
        self.add(DATA_KEY, dict())
        
    def add(self, key, value):
        """
        Adds the value to the context using the key
        
        :param key: string
            key to use
        :param value: object
            the value to store
            
        """
        self._data[key] = value
        
    def remove(self, key):
        """
        Removes the value from the context
        
        :param key: string
            key to remove from the context
        """
        assert key != None
        del(self._data[key])
        
    def contains(self, key):
        """
        Checks if the key is in the context
        
        :param key: string
            key to check
            
        :return: True if the key is in the context
        """
        return key in self._data
    
    def get(self, key, default=None):
        """
        Returns the value stored in the context at the key or the default value in the 
        context doesn't contain the key
        
        :param key: string
            key to use
        :param default: string
            the default value to use if the key is not available
        """
        if(self.contains(key)):
            return self._data[key]
        
        return default
        
    def getParams(self):
        """
        Returns the currently processed parameters
        
        :return: The param of this context
        """
        return self.get(PARAMS_KEY)
    
    def getParent(self):
        """
        Returns the parent
        
        :return: The parent chain of this context
        """
        return self.get(PARENT_KEY)
    
    def getData(self):
        """
        Returns the data
        
        :return: The data of this context
        """
        return self.get(DATA_KEY)
        


#!/usr/bin/env python
"""
Calls the CambWrapper and WmapWrapper

"""
from __future__ import print_function, division, absolute_import, unicode_literals

from wmap3Wrapper import WmapLikelihoodModule as wmap3
from wmap5Wrapper import WmapLikelihoodModule as wmap5
from wmap7Wrapper import WmapLikelihoodModule as wmap7
from wmap9Wrapper import WmapLikelihoodModule as wmap9
from pycambWrapper import PyCambCoreModule
from cambWrapper import CambCoreModule

from cosmoHammer.ChainContext import ChainContext


pyCambCore = PyCambCoreModule.PyCambCoreModule()
cambCore = CambCoreModule()
wmap3Likelihood = wmap3.WmapLikelihoodModule()
wmap5Likelihood = wmap5.WmapLikelihoodModule()
wmap7Likelihood = wmap7.WmapLikelihoodModule()
wmap9Likelihood = wmap9.WmapLikelihoodModule()
#setting up wrappers
cambCore.setup()
pyCambCore.setup()

wmap3Likelihood.setup()
wmap5Likelihood.setup()
wmap7Likelihood.setup()
wmap9Likelihood.setup()

#camb wrapper
print("setup done. calling CAMB power spectrum computation")

paramValues = [70, 0.0226, 0.122, 2.1E-009, 0.96, 0.09]

ctx = ChainContext(None, paramValues)
cambCore(ctx)

print("cls computation done. calling WMAP likelihood")
print("Likelihood WMAP 3: %s"%wmap3Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 5: %s"%wmap5Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 7: %s"%wmap7Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 9: %s"%wmap9Likelihood.computeLikelihood(ctx))

#pycamb
ctx = ChainContext(None, paramValues)
pyCambCore(ctx)

print("cls computation done. calling WMAP likelihood")
print("Likelihood WMAP 3: %s"%wmap3Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 5: %s"%wmap5Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 7: %s"%wmap7Likelihood.computeLikelihood(ctx))
print("Likelihood WMAP 9: %s"%wmap9Likelihood.computeLikelihood(ctx))




# Copyright (C) 2014 ETH Zurich, Institute for Astronomy

'''
Created on Jul 23, 2014

author: jakeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals
from IPython import parallel

class IPythonPoolWrapper(object):
    """
    Wraps the IPython balanced view to interface the Pool.map
    """
    
    def __init__(self):
        client = parallel.Client()
        self.view = client.load_balanced_view()
        
    def map(self, func, sequence):    
        return self.view.map_sync(func, sequence)
    
    def close(self):
        self.view.close()




from cosmoHammer import CosmoHammerSampler

from cosmoHammer.util.SampleFileUtil import SampleFileUtil
from cosmoHammer.util.MpiUtil import MpiPool, mpiBCast

class MpiCosmoHammerSampler(CosmoHammerSampler):
    """
    A sampler implementation extending the regular sampler in order to allow for distributing 
    the computation with MPI.

    :param kwargs:  
        key word arguments passed to the CosmoHammerSampler
    
    """
    def __init__(self, **kwargs):
        """
        CosmoHammer sampler implementation
        
        """
        self.pool = MpiPool(self._getMapFunction())
        self.rank = self.pool.rank
        
        super(MpiCosmoHammerSampler, self).__init__(pool=self.pool, **kwargs)
        
        
        
    def _getMapFunction(self):
        """
        Returns the build in map function
        """
        return map
    
    def createSampleFileUtil(self):
        """
        Returns a new instance of a File Util
        """
        return SampleFileUtil(self.filePrefix, self.isMaster(), reuseBurnin=self.reuseBurnin)
    
       
    def sampleBurnin(self, p0):
        """
        Starts the sampling process. The master node (mpi rank = 0) persists the result to the disk
        """
        p0 = mpiBCast(p0)
        
        self.log("MPI Process rank "+ str(self.rank)+" starts sampling")
        return super(MpiCosmoHammerSampler, self).sampleBurnin(p0);
   
    def sample(self, burninPos, burninProb, burninRstate, datas):
        """
        Starts the sampling process. The master node (mpi rank = 0) persists the result to the disk
        """
        burninPos = mpiBCast(burninPos)
        burninProb = mpiBCast(burninProb)
        burninRstate = mpiBCast(burninRstate)
        
        self.log("MPI Process rank "+ str(self.rank)+" starts sampling")
        super(MpiCosmoHammerSampler, self).sample(burninPos, burninProb, burninRstate, datas);

            
    def loadBurnin(self):
        """
        loads the burn in form the file system
        """
        if(self.isMaster()):
            pos, prob, rstate = super(MpiCosmoHammerSampler, self).loadBurnin()
        else:
            pos, prob, rstate = []
            
        pos = mpiBCast(pos)
        prob = mpiBCast(prob)
        rstate = mpiBCast(rstate)
        
        self.log("loading done")
        return pos, prob, rstate
    
    def createInitPos(self):
        """
        Factory method to create initial positions
        """   
        #bcast the positions to ensure that all mpi nodes start at the same position
        return mpiBCast(super(MpiCosmoHammerSampler, self).createInitPos())


    def isMaster(self):
        """
        Returns true if the rank is 0
        """
        return self.pool.isMaster()


from cosmoHammer.modules.RosenbrockModule import RosenbrockModule
from cosmoHammer.modules.MultivarianteGaussianModule import MultivarianteGaussianModule
from cosmoHammer.modules.PseudoCmbModule import PseudoCmbModule



'''
Created on Dec 2, 2013

@author: jakeret
'''
import py
import subprocess
def test_linkcheck(tmpdir):
    doctrees = tmpdir.join("doctrees")
    htmldir = tmpdir.join("html")
    subprocess.check_call(
        ["sphinx-build", "-blinkcheck",
          "-d", str(doctrees), "./source", str(htmldir)])
    
def test_build_docs(tmpdir):
    doctrees = tmpdir.join("doctrees")
    htmldir = tmpdir.join("html")
    subprocess.check_call([
        "sphinx-build", "-bhtml",
          "-d", str(doctrees), "./source", str(htmldir)])



import numpy as np


class SampleBallPositionGenerator(object):
    """
        Generates samples in a very thight n-dimensional ball 
    """
    
    def setup(self, sampler):
        """
            setup the generator
        """
        self.sampler = sampler
    
    def generate(self):
        """
            generates the positions
        """
        
        return [self.sampler.paramValues+np.random.normal(size=self.sampler.paramCount)*self.sampler.paramWidths for i in range(self.sampler.nwalkers)]
    
    def __str__(self, *args, **kwargs):
        return "SampleBallPositionGenerator"



from copy import copy
import numpy as np

class InMemoryStorageUtil(object):
	"""
	Stores the data in memory
    
    :var samplesBurin: samples from the burn in
    :var probBurnin: likelihoods from the burn in
    :var samples: samples from the burn in
    :var prob: likelihoods from the burn in
	
	
	:param master: True if the sampler instance is the master
	
	"""
	
	def __init__(self, master=True):
		if(master):
			self.samplesBurnin = None
			self.probBurnin = None
			
			self.samples = None
			self.prob = None
	
	def importFromFile(self, filePath):
		print("InMemoryStorageUtil does not support importFromFile")
		return None

	def storeRandomState(self, filePath, randomState):
		print("InMemoryStorageUtil does not support storeRandomState")

	def importRandomState(self, filePath):
		print("InMemoryStorageUtil does not support importRandomState")
		return None

	def persistBurninValues(self, pos, prob, data):
		if(self.samplesBurnin is None):
			self.samplesBurnin = copy(pos)
		else:
			self.samplesBurnin = np.append(self.samplesBurnin, pos, axis=0)
		
		if(self.probBurnin is None):
			self.probBurnin = prob
		else:	
			self.probBurnin = np.append(self.probBurnin, prob, axis=0)

		
	def persistSamplingValues(self, pos, prob, data):
		if(self.samples is None):
			self.samples = copy(pos)
		else:
			self.samples = np.append(self.samples, pos, axis=0)
		
		if(self.prob is None):
			self.prob = prob
		else:	
			self.prob = np.append(self.prob, prob, axis=0)

	def __str__(self, *args, **kwargs):
		return "InMemoryStorageUtil"


import itertools
import os
from cosmoHammer import getLogger
import time

# If mpi4py is installed, import it.
try:
    from mpi4py import MPI
    MPI = MPI
except ImportError:
    MPI = None

class MpiPool(object):
    """
    Implementation of a mpi based pool. Currently it supports only the map function.
    
    :param mapFunction: the map function to apply on the mpi nodes
    
    """
    def __init__(self, mapFunction):
        self.rank = MPI.COMM_WORLD.Get_rank()
        self.mapFunction = mapFunction
    
    def map(self, function, sequence):
        """
        Emulates a pool map function using Mpi.
        Retrieves the number of mpi processes and splits the sequence of walker position 
        in order to allow each process its block
        
        :param function: the function to apply on the items of the sequence
        :param sequence: a sequence of items
        
        :returns sequence: sequence of results
        """
        
        (rank,size) = (MPI.COMM_WORLD.Get_rank(),MPI.COMM_WORLD.Get_size())
        #sync
        sequence = mpiBCast(sequence)
        
        getLogger().debug("Rank: %s, pid: %s MpiPool: starts processing iteration" %(rank, os.getpid()))
        #split, process and merge the sequence
        mergedList = mergeList(MPI.COMM_WORLD.allgather(
                                                  self.mapFunction(function, splitList(sequence,size)[rank])))
        getLogger().debug("Rank: %s, pid: %s MpiPool: done processing iteration"%(rank, os.getpid()))
#         time.sleep(10)
        return mergedList
    
    def isMaster(self):
        """
        Returns true if the rank is 0
        """
        return (self.rank==0)
    
def mpiBCast(value):
    """
    Mpi bcasts the value and Returns the value from the master (rank = 0).
    """
    getLogger().debug("Rank: %s, pid: %s MpiPool: bcast", MPI.COMM_WORLD.Get_rank(), os.getpid())
    return MPI.COMM_WORLD.bcast(value)

def splitList(list, n):
    """
    Splits the list into block of equal sizes (listlength/n)
    
    :param list: a sequence of items
    :param n: the number of blocks to create
    
    :returns sequence: a list of blocks
    """
    getLogger().debug("Rank: %s, pid: %s MpiPool: splitList", MPI.COMM_WORLD.Get_rank(), os.getpid())
    blockLen = len(list) / float(n)
    return [list[int(round(blockLen * i)): int(round(blockLen * (i + 1)))] for i in range(n)]    

def mergeList(lists):
    """
    Merges the lists into one single list
    
    :param lists: a list of lists
    
    :returns list: the merged list
    """
    getLogger().debug("Rank: %s, pid: %s MpiPool: mergeList", MPI.COMM_WORLD.Get_rank(), os.getpid())
    return list(itertools.chain(*lists))






from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np
from cosmoHammer import getLogger

class MultivarianteGaussianModule(object):
    """
    Chain for computing the likelihood of a multivariante gaussian distribution
    """
    def __init__(self, icov, mu):
        self.icov = icov
        self.mu = mu
    
    
    def computeLikelihood(self, ctx):
        x = ctx.getParams()
        diff = x-self.mu
        return -np.dot(diff,np.dot(self.icov,diff))/2.0

    def setup(self):
        getLogger().info("Multivariante Gaussian setup")
            
        
    


#!/usr/bin/env python
"""
Runs CosmoHammer with a likelihood module simulating the WMAP likelihood by assuming the parameter distributions to be gaussian.
Yields results very similar to ones gathered using CAMB and WMAP in default config, but only needs a fraction of the time.
"""
from __future__ import print_function, division, absolute_import, unicode_literals

import numpy as np

from cosmoHammer import LikelihoodComputationChain
from cosmoHammer import CosmoHammerSampler
from cosmoHammer.util import InMemoryStorageUtil
from cosmoHammer.util import Params

from cosmoHammer.modules import PseudoCmbModule
from cosmoHammer.pso.ParticleSwarmOptimizer import ParticleSwarmOptimizer

#parameter start center, min, max, start width
params = Params(("hubble",                [70, 65, 80, 3]),
                ("ombh2",                 [0.0226, 0.01, 0.03, 0.001]),
                ("omch2",                 [0.122, 0.09, 0.2, 0.01]),
                ("scalar_amp",            [2.1e-9, 1.8e-9, 2.35e-9, 1e-10]),
                ("scalar_spectral_index", [0.96, 0.8, 1.2, 0.02]),
                ("re_optical_depth",      [0.09, 0.01, 0.1, 0.03]),
                ("sz_amp",                [1,0,2,0.4]))

chain = LikelihoodComputationChain(params[:,1], params[:,2])
chain.params = params

chain.addLikelihoodModule(PseudoCmbModule())
chain.setup()

# find the best fit value and update our params knowledge
print("find best fit point")
pso = ParticleSwarmOptimizer(chain, params[:,1], params[:,2])
psoTrace = np.array([pso.gbest.position.copy() for _ in pso.sample()])
params[:, 0] = pso.gbest.position

storageUtil = InMemoryStorageUtil()
sampler = CosmoHammerSampler(
                params= params, 
                likelihoodComputationChain=chain, 
                filePrefix="pseudoCmb_pso", 
                walkersRatio=50, 
                burninIterations=0, 
                sampleIterations=100,
                storageUtil=storageUtil,
                threadCount=4
                )

print("start sampling")
sampler.startSampling()
print("done!")

print("plotting")
import matplotlib.pyplot as plt
import triangle
data = storageUtil.samples
K = data.shape[1]
factor = 2.0 # size of one side of one panel
lbdim = 0.5 * factor # size of left/bottom margin
trdim = 0.2 * factor # size of top/right margin
whspace = 0.05 # w/hspace size
plotdim = factor * K + factor * (K - 1.) * whspace
dim = lbdim + plotdim + trdim

fig, ax = plt.subplots(K, K, figsize=(dim, dim), tight_layout=False)
triangle.corner(data, fig=fig, labels=params.keys)

for i in range(K):
    for j in range(i):
        ax[i, j].plot(psoTrace[:, j], psoTrace[:, i], "1-")


plt.show()


#!/usr/bin/env python
# encoding: utf-8

import sys
import os
from setuptools import find_packages

try:
    from setuptools import setup, Extension
    setup, Extension
except ImportError:
    from distutils.core import setup, Extension
    setup, Extension

from setuptools.command.test import test as TestCommand

class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        #import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


desc = open("README.rst").read()
required = ["numpy", "emcee"]
test_requires = ["mock"]

PACKAGE_PATH = os.path.abspath(os.path.join(__file__, os.pardir))

setup(
    name="cosmoHammer",
    version='0.6.0',
    author='Joel Akeret',
    author_email="jakeret@phys.ethz.ch",
    url="http://www.cosmology.ethz.ch/research/software-lab/cosmohammer.html",
    license="GPLv3",
    packages=find_packages(PACKAGE_PATH, "Tests"),
    description="Cosmological parameter estimation with the MCMC Hammer",
    long_description=desc,
    install_requires=required,
    test_requires=test_requires,
    package_data={"": ["LICENSE"],
                  'cosmoHammer': ['data/*.dat']},
    include_package_data=True,
    keywords=["CosmoHammer",
              "parameter estimation", 
              "cosmology", 
              "MCMC"],
    cmdclass = {'test': PyTest},
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Natural Language :: English',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
    ],
)



# Copyright (C) 2015 ETH Zurich, Institute for Astronomy

'''
Created on Aug 5, 2015

author: jakeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals
from cosmoHammer.util import Params
import pytest
import numpy as np
class TestParams(object):
    
    def test_attr_looup(self):
        params = Params(("attr1", 1),
                        ("attr2", 2))
        
        assert params.attr1 == 1
        assert params.attr2 == 2
        
        with pytest.raises(AttributeError):
            params.inexistent
            
    def test_get(self):
        params = Params(("attr1", 1),
                        ("attr2", 2))
        
        assert params.get("attr1") == 1
        assert params.get("attr2") == 2
        
        with pytest.raises(KeyError):
            params.get("inexistent")
        
    def test_params_access_1d(self):
        values = [1,2]
        params = Params(("attr1", values[0]),
                        ("attr2", values[1]))
        
        assert np.all(params.values == values)

    def test_params_access_2d(self):
        values = np.array([[1,2,3,4],
                           [5,6,7,8]])
        params = Params(("attr1", values[0]),
                        ("attr2", values[1]))
        
        assert np.all(params.values == values)

        
    def test_slicing(self):
        values = np.array([[1,2,3,4],
                           [5,6,7,8]])
        params = Params(("attr1", values[0]),
                        ("attr2", values[1]))
        
        assert np.all(values[0] == values[0])
        assert np.all(values[:, 1] == values[:, 1])
        
    def test_names_access(self):
        keys = ["attr1","attr2"]
        params = Params((keys[0], 1),
                        (keys[1], 2))
        
        assert np.all(params.keys == keys)
        
    def test_names_immutability(self):
        keys = ["attr1","attr2"]
        params = Params((keys[0], 1),
                        (keys[1], 2))
        
        params.keys.append("irrelevant")
        
        assert np.all(params.keys == keys)
        
        
    def test_copy(self):
        params = Params(("attr1", 1),
                        ("attr2", 2))
        
        params2 = params.copy()
        
        assert np.all(params.keys == params2.keys)
        assert np.all(params.values == params2.values)
        
    def test_str(self):
        params = Params(("attr1", 1))
        s = str(params)
        assert "=" in s
        
    def test_value_assignment(self):
        values = np.array([[1,2,3,4],
                           [5,6,7,8]])
        params = Params(("attr1", values[0]),
                        ("attr2", values[1]))

        params[:,0] = 0
        assert np.all(params[:,0] == 0)

    def test_duplicated_key(self):
        with pytest.raises(KeyError):
            _ = Params(("attr1", 1),
                            ("attr1", 2))
        
        


'''
Created on Sep 30, 2013

@author: J. Akeret
'''
from __future__ import print_function, division, absolute_import, unicode_literals
from copy import copy
from math import floor
import math
import multiprocessing
import numpy

class ParticleSwarmOptimizer(object):
    '''
    Optimizer using a swarm of particles
    
    :param func:
        A function that takes a vector in the parameter space as input and
        returns the natural logarithm of the posterior probability for that
        position.

    :param low: array of the lower bound of the parameter space
    :param high: array of the upper bound of the parameter space
    :param particleCount: the number of particles to use. 
    :param threads: (optional)
        The number of threads to use for parallelization. If ``threads == 1``,
        then the ``multiprocessing`` module is not used but if
        ``threads > 1``, then a ``Pool`` object is created and calls to
        ``lnpostfn`` are run in parallel.

    :param pool: (optional)
        An alternative method of using the parallelized algorithm. If
        provided, the value of ``threads`` is ignored and the
        object provided by ``pool`` is used for all parallelization. It
        can be any object with a ``map`` method that follows the same
        calling sequence as the built-in ``map`` function.
    
    '''


    def __init__(self, func, low, high, particleCount=25, threads=1, pool=None):
        '''
        Constructor
        '''
        self.func = func
        self.low = low
        self.high = high
        self.particleCount = particleCount
        self.threads = threads
        self.pool = pool
        
        if self.threads > 1 and self.pool is None:
            self.pool = multiprocessing.Pool(self.threads)
        
        self.paramCount = len(self.low)
        
        self.swarm = self._initSwarm()
        self.gbest = Particle.create(self.paramCount)
        
    def _initSwarm(self):
        swarm = []
        for _ in range(self.particleCount):
            swarm.append(Particle(numpy.random.uniform(self.low, self.high, size=self.paramCount), numpy.zeros(self.paramCount)))
        
        return swarm
        
    def sample(self, maxIter=1000, c1=1.193, c2=1.193, p=0.7, m=10**-3, n=10**-2):
        """
        Launches the PSO. Yields the complete swarm per iteration
        
        :param maxIter: maximum iterations
        :param c1: cognitive weight
        :param c2: social weight
        :param p: stop criterion, percentage of particles to use 
        :param m: stop criterion, difference between mean fitness and global best
        :param n: stop criterion, difference between norm of the particle vector and norm of the global best
        """
        self._get_fitness(self.swarm)
        i = 0
        while True:
            
            
            for particle in self.swarm:
                if ((self.gbest.fitness)<particle.fitness):
                    
                    self.gbest = particle.copy()
                    #if(self.isMaster()):
                        #print("new global best found %i %s"%(i, self.gbest.__str__()))
                    
                if (particle.fitness > particle.pbest.fitness):
                    particle.updatePBest()

            if(i>=maxIter):
                print("max iteration reached! stoping")
                return
            
            if(self._converged(i, p=p,m=m, n=n)):
                if(self.isMaster()):
                    print("converged after %s iterations!"%i)
                    print("best fit found: ", self.gbest.fitness, self.gbest.position)
                return

            
            for particle in self.swarm:
                 
                w = 0.5 + numpy.random.uniform(0,1,size=self.paramCount)/2
                #w=0.72
                part_vel = w * particle.velocity
                cog_vel = c1 * numpy.random.uniform(0,1,size=self.paramCount) * (particle.pbest.position - particle.position)
                soc_vel = c2 * numpy.random.uniform(0,1,size=self.paramCount) * (self.gbest.position - particle.position)
                particle.velocity = part_vel + cog_vel + soc_vel
                particle.position = particle.position + particle.velocity
            
            self._get_fitness(self.swarm)

            swarm = []
            for particle in self.swarm:
                swarm.append(particle.copy()) 
            yield swarm
            
            i+=1
        
    def optimize(self, maxIter=1000, c1=1.193, c2=1.193, p=0.7, m=10**-3, n=10**-2):
        """
        Runs the complete optimiziation.
        
        :param maxIter: maximum iterations
        :param c1: cognitive weight
        :param c2: social weight
        :param p: stop criterion, percentage of particles to use 
        :param m: stop criterion, difference between mean fitness and global best
        :param n: stop criterion, difference between norm of the particle vector and norm of the global best

        :return swarms, gBests: the swarms and the global bests of all iterations
        """
        
        swarms = []
        gBests = []
        for swarm in self.sample(maxIter,c1,c2,p,m,n):
            swarms.append(swarm)
            gBests.append(self.gbest.copy())
        
        return swarms, gBests
        
    def _get_fitness(self,swarm):
        
        # If the `pool` property of the pso has been set (i.e. we want
        # to use `multiprocessing`), use the `pool`'s map method. Otherwise,
        # just use the built-in `map` function.
        if self.pool is not None:
            mapFunction = self.pool.map
        else:
            mapFunction = map
        
        pos = numpy.array([part.position for part in swarm])
        results =  mapFunction(self.func, pos)
        lnprob = numpy.array([l[0] for l in results])
        for i, particle in enumerate(swarm):
            particle.fitness = lnprob[i]

    def _converged(self, it, p, m, n):
#        test = self._convergedSpace2(p=p)
#        print(test)
        fit = self._convergedFit(it=it, p=p, m=m)
        if(fit):
            space = self._convergedSpace(it=it, p=p, m=n)
            return space
        else:
            return False
        
    def _convergedFit(self, it, p, m):
        bestSort = numpy.sort([particle.pbest.fitness for particle in self.swarm])[::-1]
        meanFit = numpy.mean(bestSort[1:math.floor(self.particleCount*p)])
#        print( "best %f, meanFit %f, ration %f"%( self.gbest[0], meanFit, abs((self.gbest[0]-meanFit))))
        return (abs(self.gbest.fitness-meanFit)<m)
    
    def _convergedSpace(self, it, p, m):
        sortedSwarm = [particle for particle in self.swarm]
        sortedSwarm.sort(key=lambda part: -part.fitness)
        bestOfBest = sortedSwarm[0:int(floor(self.particleCount*p))]
        
        diffs = []
        for particle in bestOfBest:
            diffs.append(self.gbest.position-particle.position)
            
        maxNorm = max(list(map(numpy.linalg.norm, diffs)))
        return (abs(maxNorm)<m)

    def _convergedSpace2(self, p):
        #Andres N. Ruiz et al.
        sortedSwarm = [particle for particle in self.swarm]
        sortedSwarm.sort(key=lambda part: -part.fitness)
        bestOfBest = sortedSwarm[0:int(floor(self.particleCount*p))]
        
        positions = [particle.position for particle in bestOfBest]
        means = numpy.mean(positions, axis=0)
        delta = numpy.mean((means-self.gbest.position)/self.gbest.position)
        return numpy.log10(delta) < -3.0


    def isMaster(self):
        return True   

class Particle(object):
    """
    Implementation of a single particle
    
    :param position: the position of the particle in the parameter space
    :param velocity: the velocity of the particle
    :param fitness: the current fitness of the particle
    
    """
    
    
    def __init__(self, position, velocity, fitness = 0):
        self.position = position
        self.velocity = velocity
        
        self.fitness = fitness
        self.paramCount = len(self.position)
        self.pbest = self

    @classmethod
    def create(cls, paramCount):
        """
        Creates a new particle without position, velocity and -inf as fitness
        """
        
        return Particle(numpy.array([[]]*paramCount),
                 numpy.array([[]]*paramCount),
                 -numpy.Inf)
        
    def updatePBest(self):
        """
        Sets the current particle representation as personal best
        """
        self.pbest = self.copy()
        
    def copy(self):
        """
        Creates a copy of itself
        """
        return Particle(copy(self.position),
                        copy(self.velocity),
                        self.fitness)
        
    def __str__(self):
        return "%f, pos: %s velo: %s"%(self.fitness, self.position, self.velocity)
    
    def __unicode__(self):
        return self.__str__()


from __future__ import print_function, division, absolute_import, unicode_literals
from cosmoHammer import getLogger

class RosenbrockModule(object):
    """
    A module for the computation of the rosenbrock likelihood
    """
    
    def __init__(self):
        self.a1 = 100.0
        self.a2 = 20.0

    def computeLikelihood(self, ctx):
        p = ctx.getParams()
        return -(self.a1 * (p.y - p.x**2)**2 + (1 - p.x)**2) / self.a2
    
    def setup(self):
        getLogger().info("Rosenbrock setup")


class DummyCoreModule(object):
    """
    Dummy Core Module for calculating the squares of parameters.
    """

    def __init__(self):
        """
        Constructor of the DummyCoreModule
        """
        pass
        
    def __call__(self, ctx):
        """
        Computes something and stores it in the context
        """
        # Get the parameters from the context
        p = ctx.getParams()

        # Calculate something
        squares = p**2
        # Add the result to the context using a unique key
        ctx.add('squares_key', squares)
        
        # Store derived parameters for post processing
        derived_parms = sum(squares) % 2
        ctx.getData()["derived_params_key"] = derived_parms

    def setup(self):
        """
        Sets up the core module.
        Tasks that need to be executed once per run
        """
        #e.g. load data from files
        
        print("DummyCoreModule setup done")



import pickle
import numpy as np
import cosmoHammer.Constants as c

class SampleFileUtil(object):
	"""
	Util for handling sample files
	
	:param filePrefix: the prefix to use
	:param master: True if the sampler instance is the master
	:param  reuseBurnin: True if the burn in data from a previous run should be used
	
	"""
	
	def __init__(self, filePrefix, master=True, reuseBurnin=False):
		self.filePrefix = filePrefix
		
		if(master):
			if(reuseBurnin):
				mode = "r"
			else:
				mode = "w"
			self.samplesFileBurnin = open(self.filePrefix+c.BURNIN_SUFFIX, mode)
			self.probFileBurnin = open(self.filePrefix+c.BURNIN_PROB_SUFFIX, mode)
			
			self.samplesFile = open(self.filePrefix+c.FILE_SUFFIX, "w")
			self.probFile = open(self.filePrefix+c.PROB_SUFFIX, "w")
	
	def importFromFile(self, filePath):
		values = np.loadtxt(filePath, dtype=float)
		return values

	def storeRandomState(self, filePath, randomState):
		with open(filePath,'wb') as f:
			pickle.dump(randomState, f)

	def importRandomState(self, filePath):
		with open(filePath,'rb') as f:
			state = pickle.load(f)
		return state

	def persistBurninValues(self, pos, prob, data):
		self.persistValues(self.samplesFileBurnin, self.probFileBurnin, pos, prob, data)
		
	def persistSamplingValues(self, pos, prob, data):
		self.persistValues(self.samplesFile, self.probFile, pos, prob, data)
		

	def persistValues(self, posFile, probFile, pos, prob, data):
		"""
		Writes the walker positions and the likelihood to the disk
		"""
		posFile.write("\n".join(["\t".join([str(q) for q in p]) for p in pos]))
		posFile.write("\n")
		posFile.flush()
		
		probFile.write("\n".join([str(p) for p in prob]))
		probFile.write("\n")
		probFile.flush();
		
	def close(self):
		self.samplesFileBurnin.close()
		self.probFileBurnin.close()
		self.samplesFile.close()
		self.probFile.close()

	def __str__(self, *args, **kwargs):
		return "SampleFileUtil"



import multiprocessing
from cosmoHammer.MpiCosmoHammerSampler import MpiCosmoHammerSampler


class ConcurrentMpiCosmoHammerSampler(MpiCosmoHammerSampler):
    """
    A sampler implementation extending the mpi sampler in order to allow to 
    distribute the computation with MPI and using multiprocessing on a single node.

    :param threads:  (optional)
        The number of threads to use for parallelization. If ``threads == 1``,
        then the ``multiprocessing`` module is not used but if
        ``threads > 1``, then a ``Pool`` object is created
        
    :param kwargs: key word arguments passed to the CosmoHammerSampler

    """
    def __init__(self, threads=1, **kwargs):
        """
        CosmoHammer sampler implementation
        
        """
        
        self.threads = threads
        
        super(ConcurrentMpiCosmoHammerSampler, self).__init__(**kwargs)
        
        
    def _getMapFunction(self):
        if self.threads > 1:
            pool = multiprocessing.Pool(self.threads)
            return pool.map
        else:
            return map



"""
Test the CosmoHammerSampler module.

Execute with py.test -v

"""
from __future__ import print_function, division, absolute_import, unicode_literals

from cosmoHammer.ChainContext import ChainContext

import numpy as np

class TestCosmoHammerSampler(object):
    ctx = None
    params = np.array([[1,2,3],[4,5,6]])
    
    def setup(self):
        self.ctx=ChainContext(self, self.params)
        
    def test_no_data(self):
        assert self.ctx.getParent() == self
        assert self.ctx.getParams() is self.params
        assert self.ctx.contains("key") == False
        assert self.ctx.getData() is not None
        assert len(list(self.ctx.getData().items())) == 0
    
    def test_add_data(self):
        assert self.ctx.contains("key") == False
        assert self.ctx.getData() is not None
        
        self.ctx.add("key", "value")
        
        assert self.ctx.contains("key") == True
        assert self.ctx.getData() is not None
        
        assert self.ctx.get("key") == "value"

    def test_remove_data(self):
        assert self.ctx.contains("key2") == False
        assert self.ctx.getData() is not None
        
        self.ctx.add("key2", "value")
        
        assert self.ctx.contains("key2") == True
        assert self.ctx.getData() is not None
        
        assert self.ctx.get("key2") == "value"
        
        self.ctx.remove("key2")
        assert self.ctx.contains("key2") == False

    def test_get_default(self):
        assert self.ctx.contains("aaa") == False
        assert self.ctx.get("aaa", "default") == "default"
        
        
    def test_additional_data(self):
        assert self.ctx.getData() is not None
        assert len(list(self.ctx.getData().items())) == 0
        
        self.ctx.getData()["moreData"] = "data"
        assert len(list(self.ctx.getData().items())) == 1